@@ -19,6 +19,9 @@ $svg->rect('width' => 300, 'height' => 300, 'fill' => 'white');
 # baseline test
 $svg->text('x' => 30, 'y' => 20, 'fill' => 'red')->cdata('Hello World');
 $svg->line(x1 => 30, y1 => 20, x2 => 104, y2 => 20, 'stroke' => 'black');
+$g = $svg->group(transform => 'translate(30, 20) rotate(15)');
+$g->text('fill' => 'blue')->cdata('Hello World');
+$g->line(x1 => 0, y1 => 0, x2 => 74, y2 => 0, 'stroke' => 'black');
 
 # deferred rasterization, flush in right order, text carries on
 $text = $svg->text('x' => 30, 'y' => 60);
@@ -55,6 +58,21 @@ $text = $svg->text(x => 150, 'y' => '50 52 54 56 58 60');
 $node = $text->tspan('font-family' => 'Verdana');
 $node->cdata('stairs');
 
+# two rotate examples, taken from SVG spec
+$text = $svg->text(x => 150, 'y' => 90, 'font-family' => 'Verdana');
+$text->tspan('rotate' => '-30,0,30')->cdata('Hello, out there');
+
+# here we cannot rebuild it exactly due to limitations in SVG
+$g    = $svg->group(transform => 'translate(150, 120)');
+$text = $g->text('font-family' => 'Arial, sans-serif',
+		 'fill'        => 'red',
+		 'rotate'      => '5 15 25 35 45 55');
+$text->cdata('Not ');
+$node = $text->tspan('rotate' => "-10,-20,-30,-40",
+		     'fill'   => "orange")->cdata('all characters ');
+my $node2 = $node->tspan('rotate' => "70,60,50,40,30,20,10",
+			 'fill'   => "yellow")->cdata('in ');
+
 $rasterize = SVG::Rasterize->new;
 $rasterize->rasterize(width => 300, height => 300, svg => $svg);
 
